/* Project name:
      Mulitasking
 * Copyright:
     (c) -
 * Created by:
      Stevan A. Milinkovic
      Union University, School of Computing
      Knez Mihailova 6, 11000 Belgrade, Serbia
      e-mail: smilinkovic@raf.edu.rs
 * Revision History:
       - 2012-07-19 v 0.0.0.4
       
 * Description:
     Multitasking example for the world of ARM MCUs using FreeRTOS;
 * Test configuration:
     Device:          STM32F107VC
                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
     Dev.Board:       EasyMx v7 for STM(R) ARM(R)
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HSE-PLL, 72.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/eng/products/view/752/mikroc-pro-for-arm/
 * NOTES:
     - Turn ON PORTA, PORTB, PORTC, PORTD, PORTE at SW15 (board specific).
     - Turn on SW11.3 (display back light) to SW11.8 (board specific)
     - Turn on SPI and audio switches SW13.1 to SW13.7 and turn on microSD card
       switches SW14.3 and SW14.4
 */


#include "FreeRTOS.h"
#include "task.h"
#include "calculator_objects.h"
#include "queue.h"
#include "timers.h"

#define ledSTACK_SIZE               configMINIMAL_STACK_SIZE
#define mainBLINKING_TASK_PRIORITY  ( tskIDLE_PRIORITY + 3 )
#define CALC_TASK_PRIORITY          ( tskIDLE_PRIORITY + 2 )
#define AUDIO_TASK_PRIORITY          CALC_TASK_PRIORITY
#define TaskDelay_ms(x)             vTaskDelay( x/portTICK_RATE_MS )


void MP3_Start(void);
void MP3_Init(void);
void InitMCU(void);
void MP3_Test(char *test);

extern char ucMP3_run_test;

static portTASK_FUNCTION( vCalculatorTask, pvParameters ) {
  for (;;) {
    Check_TP();
    taskYIELD();
  }
}

static portTASK_FUNCTION( vLed1BlinkingTask, pvParameters ) {
  for (;;) {
    GPIOA_ODR.B0 = ~GPIOA_ODR.B0;     // Toggle PORTA.0
    TaskDelay_ms(500);
  }
}

static portTASK_FUNCTION( vLed2BlinkingTask, pvParameters ) {
  for (;;) {
    GPIOA_ODR.B4 = ~GPIOA_ODR.B4;    // Toggle PORTA.4
    TaskDelay_ms(500);
  }
}

static portTASK_FUNCTION( vAudioTask, pvParameters ) {
    InitMCU();
    MP3_Start();
    MP3_Test(&ucMP3_run_test);
    vTaskDelete( NULL );             // Task suicide
}


void main(void ) {
    Start_TP();
    
    GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_0); // Set PORTA.0  as digital output
    GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_4); // Set PORTA.4  as digital output
    GPIOA_ODR.B0 = 0;
    GPIOA_ODR.B4 = 1;

    xTaskCreate( vCalculatorTask,  ( signed char * ) "CALC", ledSTACK_SIZE, NULL, CALC_TASK_PRIORITY, ( xTaskHandle * ) NULL );
    xTaskCreate( vLed1BlinkingTask, ( signed char * ) "LED1",  ledSTACK_SIZE, NULL, mainBLINKING_TASK_PRIORITY, ( xTaskHandle * ) NULL );
    xTaskCreate( vLed2BlinkingTask, ( signed char * ) "LED2",  ledSTACK_SIZE, NULL, mainBLINKING_TASK_PRIORITY, ( xTaskHandle * ) NULL );
    xTaskCreate( vAudioTask, ( signed char * ) "AUDIO",  ledSTACK_SIZE, NULL, AUDIO_TASK_PRIORITY, ( xTaskHandle * ) NULL );
    vTaskStartScheduler();
}